use aead_vfs::open_dir;
use std::{
    io::{Read, Write},
    process::exit,
};
use structopt::StructOpt;

/// Read some lines of a file
#[derive(Debug, StructOpt)]
struct Cli {
    /// Enable Debug Output
    #[structopt(short, long)]
    debug: bool,
    /// Directory to store
    path: String,
    /// The passphrase for the encryption
    passphrase: String,
    /// The encrypted file
    secure_file_path: String,
    /// Actions: create, read, store, delete, exists
    action: Action,
    /// content for create and store
    content: Option<String>,
}

#[derive(Debug)]
enum Action {
    Create,
    Read,
    Store,
    Delete,
    Exists,
}
impl std::str::FromStr for Action {
    type Err = String;
    fn from_str(s: &str) -> Result<Self, Self::Err> {
        use Action::*;
        let action = s.trim().to_lowercase();
        Ok(match &*action {
            "create" => Create,
            "read" => Read,
            "store" => Store,
            "delete" => Delete,
            "exists" => Exists,
            unknown => return Err(format!("unknown Action: {unknown:?}")),
        })
    }
}

fn main() {
    let cli = Cli::from_args();
    println!("{cli:#?}");

    let mut vfs = open_dir(&cli.passphrase, cli.path).expect("unable to open");

    if cli.debug {
        println!("    ======================");
        println!("{vfs:#?}");
        println!("    ======================");
    }

    use Action::*;
    match (cli.action, cli.content) {
        (Create, None) | (Store, None) => {
            eprintln!("Must provide content argument");
            exit(23);
        }
        (Create, Some(content)) => {
            let mut a = vfs.create(&cli.secure_file_path).unwrap();
            a.set_len(content.len()).unwrap();

            a.write_all(content.as_bytes()).unwrap();
        }
        (Read, _) => {
            if vfs.exists(&cli.secure_file_path).unwrap() == false {
                eprintln!("secure_file_path {:#?} not found", cli.secure_file_path);
                exit(23);
            }
            let mut fd = vfs.create(&cli.secure_file_path).unwrap();
            let mut buffer = String::new();
            fd.read_to_string(&mut buffer).unwrap();
            println!("{buffer}");
        }
        (Store, Some(content)) => {
            let mut fd = vfs.create(&cli.secure_file_path).unwrap();
            fd.write_all(content.as_bytes()).unwrap();
        }
        (Delete, _) => {
            let _r = vfs.delete(&cli.secure_file_path).unwrap();
        }
        (Exists, _) => {
            let r = vfs.exists(&cli.secure_file_path).unwrap();
            if r {
                println!("exists");
            } else {
                println!("absent");
            }
        }
    }
}
